#pragma once
#include <vector>
#include <random>

#include "CoreMinimal.h"
#include "Tiles.h"
#include "CitySettings.h"
#include "ProceduralMeshComponent.h"

class CROWDSIMULATION_API CityMap
{
public:
	CityMap();
	~CityMap();

	//generate city tiles and mesh according to supplied texture
	void generate(const CitySettings& settings);

	const std::vector<Tiles*>& getBuildings() const;
	
	Tiles* getTileForLocation(const FVector loc) const;
	Tiles* getRandomStreetTile();
	Tiles* getNearestTile(Tiles* tile, TileType type) const;

	void emptyOccupants();
	void getOccupantsForPosition(std::vector<Agent*>& occupants, const FVector& pos, float radius);

	FVector wallsAvoidanceForce(const FVector& pos, const FVector& dir, const float agentRadius, const float epsilon);
	FVector wallsRepulsionForce(const FVector& pos, const float radius, const float epsilon);

	int getRandomWaitInterval();
	
private:
	void generateCity(const CitySettings& settings);
	void generateMesh(const CitySettings& settings);

	int32 resolution_x;
	int32 resolution_y;
	int32 res;
	int32 spacing;
	int32 height;
		
	std::vector<Tiles*> tiles;
	std::vector<Tiles*> building;
	std::vector<Tiles*> food;
	std::vector<Tiles*> street;

	std::default_random_engine generator;
	std::uniform_int_distribution<int> randomStreet;
	std::uniform_int_distribution<int> randomWaitInterval;
};
