#pragma once
#include <vector>

#include "CoreMinimal.h"

class CROWDSIMULATION_API Agent;

enum TileType {
	Border,
	Building,
	Street,
	Food
};

class CROWDSIMULATION_API Tiles
{
public:
	Tiles(int x, int y, int w, int h, int z);
	virtual ~Tiles() {};

	FVector getLocation() const;

	//returns number of vertices generated
	virtual int generate_mesh(TArray<FVector>& vertices, TArray<FVector>& normals, TArray<int32>& indices, TArray<FVector2D>& uvs, const int index_offset, TArray<FLinearColor>& color) = 0;

	virtual FVector nearestWallNormal(const FVector& pos) const;
	const std::vector<Tiles*>& getNeighbors() const;

	void setup_neighbors(std::vector<Tiles*> tiles, int32 resolution_x, int32 resolution_y);

	void emptyOccupants();
	void addOccupant(Agent* agent);
	void getOccupants(std::vector<Agent*>& agents) const;

	FVector getVecToClosestTilePoint(const FVector& pos);

	virtual int getTileType() = 0;

protected:
	static int generateBox(TArray<FVector>& vertices, TArray<FVector>& normals, TArray<int32>& indices, TArray<FVector2D>& uvs, const int index_offset, Tiles& tile);

	int x, y, pos_x, pos_y, w, h, z;

	std::vector<Tiles*> neighbors;
	std::vector<Agent*> occupants;
};
